// SCalculator.cpp: Implementierung der Klasse SCalculator.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <iostream.h>
#include <math.h>
#include "SCalculator.h"

//*******************
//Defines/Constants
//*******************

#define SAFE_DELETE_ARRAY(a) if(a){delete[] a;}
#define INIT_TOKEN_ARRAY(a) for(int jbc=0; jbc < MAX_TOKEN;jbc++)  {a[jbc].type=NONE;a[jbc].value="";}

#define PI 3.14159

//State vars
#define ANGLE_STATE "rad"

//Conversion constants

const double RAD2DEG = 180/PI;
const double DEG2RAD = PI/180;

//*******************
//Searcorder
//*******************

//!!!increment !!!
#define NUM_OPERS 10
const SType searchOrder[] = { LP,ABS,FUNC,POW,MUL,DIV,DIV2,PLUS,MINUS,IS};

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

SCalculator::SCalculator()
{
	size = 0;
	varList = NULL;

	//Predefined variables
	addVariable("pi",PI);
	
	//State vars
	addVariable(ANGLE_STATE,0.0);
}

SCalculator::~SCalculator()
{
	if (varList != NULL)

	{
		delete varList;
	}
}

double SCalculator::parse(SString &expression)
{
	INIT_TOKEN_ARRAY(tokens)

	int position = -1;
	int last = -1;
	size = MAX_TOKEN;

	//*******************
	//pre-parsing
	//*******************
	//lowercase
	expression.toLowerCase();

	//trim
	expression.remove(' ');
	expression.remove('\t');

	//'[{..' are supported
	expression.replace('{','(');
	expression.replace('[','(');

	expression.replace('}',')');
	expression.replace(']',')');

	//********************
	//parsing
	//********************

	//parse expression
	for (int i = 0; i  < expression.getLength(); i += 1)
	{
		//variable	
		if (expression[i] >= 'a' && expression[i] <= 'z')
		{
			//next token
			if (last != NAME || last == NONE)
				position++;
			
			tokens[position].type = NAME;
			tokens[position].value += expression[i];
			last = NAME;
		}
		//number
		else if ((expression[i] >= '0' && expression[i] <= '9') ||
				expression[i] == '.')
		{
			if (last != NUMBER || last == NONE)
				position++;

			tokens[position].type = NUMBER;
			tokens[position].value += expression[i];
			last = NUMBER;
		}
		//operator
		else
		{
			position += 1;
			tokens[position].type = SType(expression[i]);
			
			last = NONE;
			 //next token
			//size += 2; //oper + previous token(number)
		}
	}

	//after initial parse
	postParse();

	return calculate();
}

double SCalculator::calculate(int begin,int end)
{
	int index = -1;

	for(int i=0; i < NUM_OPERS; i++)
	{
		while ( (index = getNextOper(searchOrder[i],begin,end)) != -1 )
		{	
			//cout << "Oper pos:" << index << endl;
			if (searchOrder[i] == LP)
			{
				int start = index+1;
				int finish = getNextEndToken(RP,start,end);
				
				double result = calculate(start,finish);

				tokens[index].value = result;
				tokens[index].type = NUMBER;

				tokens[finish].type = NONE;
			}
			else if (searchOrder[i] == ABS)
			{
				int start = index+1;
				int finish = getNextEndToken(ABS,start,end);


				double result = fabs(calculate(start,finish));

				//cout << "ABS result:" << result << endl;

				tokens[index].value = result;
				tokens[index].type = NUMBER;

				tokens[finish].type = NONE;
			}
			else if (searchOrder[i] == FUNC)
			{
				int start = index+1;
				int finish = getNextEndToken(RP,start,end);
				
				double result = processFunction(tokens[index].value,calculate(
							start,finish));

				tokens[index].value = result;
				tokens[index].type = NUMBER;

				tokens[finish].type = NONE;	
			}
			else
			{
				///*DEBUG*/cout << "Oper pos:" << index << endl;
				
				tokens[index].value = getResult(index);
				tokens[index].type = NUMBER;
			}
		}
	}

	//delete spaces
	//trimDown();
	
	if (begin == -1)
		begin = 0;

	//the one and only number...
	return getNextNumber(begin);
}

double SCalculator::getResult(int index)
{
	double tmp;

	//what type do you want to have today?
	switch(tokens[index].type)
	{
	case MINUS:
		return getNextNumber(index-1,true)
			-getNextNumber(index+1);
	case PLUS:
		return getNextNumber(index-1,true)+getNextNumber(index+1);
	case MUL:
		return getNextNumber(index-1,true)*getNextNumber(index+1);
	case DIV:
	case DIV2:
		if ((tmp = getNextNumber(index+1)) != 0.0)
		{
			return getNextNumber(index-1,true)/tmp;
		}
		else
		{
			cerr << "Error: Divide by zero!" << endl;
			return 0.0;
		}
	case  POW:
		return pow(getNextNumber(index-1,true),getNextNumber(index+1));
	case IS: // name=number
		tmp = getNextNumber(index);
		addVariable(getNextName(index),tmp);
		return tmp;
	}

	//hmmm something must be wrong..
	return 0.0;
}

int SCalculator::getNextOper(SType oper, int begin,int end,bool reverse)
{
	if (reverse)
	{
		for (int i = (end==-1?size-1:end-1); i >= (end==-1?0:begin); i--)
		{
			if (tokens[i].type == oper)
			{
				return i;
			}
		}
	}
	else
	{
		for (int i = (begin==-1?0:begin); i < (end==-1?size:end); i++)
		{
			if (tokens[i].type == oper)
			{
				return i;
			}
		}
	}

	//and then there were none..
	return -1;
}

double SCalculator::getNextNumber(int begin,bool reverse)
{
	//delete numbers at the same time
	// REMEMBER: When calling this function the next number
	//	     will be immediately lost. So save it!!!
	if (reverse)
	{
		for (int i = begin; i >= 0; i--)
		{

			if (tokens[i].type == NUMBER)
			{
				//del. nbr.
				tokens[i].type = NONE;
				return tokens[i].value.toDouble();
			}
			else if (tokens[i].type == LP || tokens[i].type == ABS)
			{
				//out of ()'s or ||'s
				return 0;
			}
		}
			
	}
	else
	{
		for (int i = begin; i < size; i++)
		{
			if (tokens[i].type == NUMBER)
			{
				tokens[i].type = NONE;
				return tokens[i].value.toDouble();
			}
			else if (tokens[i].type == RP || tokens[i].type == ABS)
			{
				//out of ()'s or ||'s
				return 0;
			}
		}
	}

	//and then there were none..
	return 0;
}

void SCalculator::postParse()
{
	//GOALS:
	// 1.Unary minus -> -number -> unary number gets +
	// 2.Look for functions
	// 3.Variables -> Translate into numbers

	for (int i = 0; i < size-1; i++)
	{
		//1.
		if (tokens[i].type == MINUS && tokens[i+1].type == NUMBER)
		{
			tokens[i+1].value = -tokens[i+1].value.toDouble();
			tokens[i].type = PLUS;
		}

		//2.
		// ex. sqrt(  -> NAME,LP
		if (tokens[i].type == NAME && tokens[i+1].type == LP)
		{
			tokens[i].type = FUNC;
			tokens[i].value += '(';
			
			tokens[i+1].type = NONE;
		}
		
		//3.
		if (tokens[i].type == NAME && tokens[i+1].type != IS)
		{
			//*************************************************
			//Test for both 2pi and pi2 (2a/a2) -> 2*pi / pi*2
			//*************************************************
			if (i != 0)
			{
				if (tokens[i-1].type==NUMBER)
				{
					tokens[i].value = getVariable(
						tokens[i].value)*tokens[i-1].value.toDouble();
					goto lastThing;

				}
			}
			
			if (tokens[i+1].type==NUMBER)
			{
				tokens[i].value = getVariable(
					tokens[i].value)*tokens[i+1].value.toDouble();
				goto lastThing;
			}
					
			tokens[i].value = getVariable(tokens[i].value);
		lastThing:
			tokens[i].type = NUMBER;
		}
		
	}
}

void SCalculator::addVariable(SString name, double value)
{	
	SVariable* newVar;
	SVariable* current;

	if ((current = variableExists(name)) != NULL)
	{
		current->value = value;
		return;
	}

	if (varList == NULL)
	{
		varList = new SVariable;
	}

	newVar = new SVariable;
	newVar->name = name;
	newVar->value = value;

	current = varList;


	while (current->next != NULL) 
	{
		current = current->next;
	}

	current->next = newVar;	
}

double SCalculator::getVariable(SString name)
{
	SVariable* current = varList;

	//go through every entry
	while (current != NULL)
	{
		//matching entry -> return value
		if (current->name == name)
		{
			return current->value;
		}
		
		current = current->next;
	}

	//should never reach
	cerr << "Error: Variable '" << name << "' is undeclared. It is used as 0.0 ." << endl;
	return 0.0f;
}

SString SCalculator::getNextName(int index)
{
	for (int i = index; i >= 0; i--)
	{
		if (tokens[i].type == NAME)
		{
			//del. name
			tokens[i].type = NONE;
			return tokens[i].value;
		}
	}

	return "";
}

SVariable* SCalculator::variableExists(SString &name)
{
	SVariable* current = varList;

	//go through every entry
	while (current != NULL)
	{
		if (current->name == name)
		{
			return current;
		}
		
		current = current->next;
	}

	return NULL;
}	

int SCalculator::getNextEndToken(SType token, int begin, int end)
{
	//counts tokens between e.g. ( ( ) ) should
	//return the last ) and not the one before the real one.
	//->therefore, tmp counts the number of these
	int tmp = 0;
	SType startToken = NONE;

	if (token == RP)
		startToken = LP;
	else if (token == ABS)
		startToken = ABS;


	for (int i = (begin==-1?0:begin); i < (end==-1?size:end); i++)
	{
		if (tokens[i].type == token)
		{
			if (!tmp)
				return i;
			else
				tmp--;
		}
		else if (tokens[i].type == startToken)
		{
			tmp++;
		}
		
	}

	return -1;
}

double SCalculator::processFunction(SString& func,double parameter)
{
	if (func == "sqrt(")
	{
		if (parameter >= 0)
			return sqrt(parameter);
		else
		{
			cerr << "Error: Parameter of sqrt() can't be negative." << endl;
			return 0;
		}
	}
	else if (func == "asin(" || func == "arcsin(")
		return 	(getVariable(ANGLE_STATE) == 0.0?
				RAD2DEG*asin(parameter):
				asin(parameter));
	else if (func == "acos(" || func == "arccos(")
		return (getVariable(ANGLE_STATE) == 0.0?
				RAD2DEG*acos(parameter):
				acos(parameter));
	else if (func == "atan(" || func == "arctan(")
		return (getVariable(ANGLE_STATE) == 0.0?
				RAD2DEG*atan(parameter):
				atan(parameter));
	else
	{
		//** trigo. functions **
				
		//convert degrees (param) -> rads
		if (getVariable(ANGLE_STATE) == 0.0)
			parameter = parameter*DEG2RAD;
			
		if (func == "sin(")
			return sin(parameter);
		else if (func == "cos(")
			return cos(parameter);
		else if (func == "tan(")
			return tan(parameter);
		else if (func == "cot(")
			return 1.0/tan(parameter);
	}

	//should never reach
	
	cerr << "Error: Undefined function '" << func << ")'. Returns 0.0 ." << endl;
	return 0;	
}

void SCalculator::printVars()
{
	SVariable* current = varList;
	
	cout << endl;

	//go through every entry
	while (current != NULL)
	{
		cout << current->name << "\t\t" << current->value << endl;
		
		current = current->next;
	}

	cout << endl;
	
}
