// SString.cpp: Implementierung der Klasse SString.
//
// Copyright by Andr Stein
// E-Mail: andre_stein@gmx.de, andre_stein@web.de
// http://www.steinsoft.net
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "SString.h"

#define SAFE_DELETE_ARRAY(a) if(a!=0){ delete[] a; a=0; }

static wchar_t NULL_CHAR = 0;

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

SString::SString()
{
	string = 0;
	buffer = 0;
	length = 0;
	size = 0;
}

SString::~SString()
{
	SAFE_DELETE_ARRAY(string)
	SAFE_DELETE_ARRAY(buffer)
}


SString::SString(wchar_t *tmp)
{
	buffer = 0;
	length = wcslen(tmp);
	size = length + 1;
	string = new wchar_t[size];

	for (int i = 0; i < size; i += 1)
	{
		string[i] = tmp[i];
	}
}

SString::SString(char *tmp)
{
	buffer = 0;
	length = strlen(tmp);
	size = length + 1;
	string = new wchar_t[size];

	for (int i = 0; i < length; i += 1)
	{
		string[i] = tmp[i];
	}
}

SString::SString(const int number)
{
	buffer = 0;
	char tmp[24];
	sprintf(tmp,"%d",number);

	length = strlen(tmp);
	size = length + 1;

	string = new wchar_t[size];

	for (int i = 0; i < size; i += 1)
	{
		string[i] = tmp[i];
	}
}

SString::SString(const double number)
{
	buffer = 0;
	char tmp[64];
	sprintf(tmp,"%f",number);

	length = strlen(tmp);
	size = length + 1;

	string = new wchar_t[size];

	for (int i = 0; i < size; i += 1)
	{
		string[i] = tmp[i];
	}
}

SString::SString(const char aChar)
{
	buffer = 0;
	length = 1;	
	size = 2;
	string = new wchar_t[size];

	string[0] = aChar;
	string[1] = '\0';
}

SString::SString(const wchar_t aChar)
{
	buffer = 0;
	length = 1;
	size = 2;
	string = new wchar_t[size];

	string[0] = aChar;
	string[1] = '\0';
}

SString::SString(const SString &aString)
{
	buffer = 0;
	length = aString.length;
	size = aString.size;
	string = new wchar_t[size];

	for (int i = 0; i < size; i += 1)
	{
		string[i] = aString.string[i];
	}
}

/////////////////////////////////////////////////////////////////////
// Funktionen
/////////////////////////////////////////////////////////////////////

int SString::getLength()
{
	return length;
}

int SString::toInteger()
{
	char* tmpBuffer = new char[size];
	int returnValue;

	for (int i = 0; i < size; i += 1)
	{
		tmpBuffer[i] = string[i];
	}

	returnValue = atoi(tmpBuffer);

	return returnValue;
}

double SString::toDouble()
{
	char* tmpBuffer = new char[size];
	double returnValue;

	for (int i = 0; i < size; i += 1)
	{
		tmpBuffer[i] = string[i];
	}

	returnValue = atof(tmpBuffer);

	return returnValue;
}

void SString::replace(SString what, SString result)
{
	bool found;
	int  oldIndex;

	for (int i = 0; i < length; i += 1)
	{
		found = true;

		for (int j = i,l = 0; j < i + what.length; j += 1, l += 1)
		{
			if (j >= length)
			{
				found = false;
				break;
			}
			
			if (operator[](j) != what[l])
			{
				found = false;
				break;
			}
		}

		if (found)
		{
			oldIndex = i;

			for (int k = i; k < i + what.length; k += 1)
			{
				deleteAt(k);
				k -= 1;
				i -= 1;
			}
			
			insertAt(oldIndex,result);
		}
	}
	
/*	int index;
	int beginAt = 0;

	while ( (index = search(what) ) != -1)
	{
		for (int i = index; i < index+what.length; i += 1)
		{
			deleteAt(i);
			i -= 1;
		}

		insertAt(index,result);
		beginAt = index+what.length;
	}*/
}

void SString::trim()
{
	trimRight();
	trimLeft();
}

void SString::trimLeft()
{
	for (int i = 0; i < length; i += 1)
	{
		if (string[i] == ' ' || string[i] == '\t' || string[i] == '\r' ||
			string[i] == '\n')
		{
			deleteAt(i);
			i -= 1;
		}
		else
		{
			break;
		}
	}
}

void SString::trimRight()
{
	for (int i = length - 1; i >= 0; i -= 1)
	{
		if (string[i] == ' ' || string[i] == '\t' || string[i] == '\r' ||
			string[i] == '\n')
		{
			deleteAt(i);
			//i += 1;
		}
		else
		{
			break;
		}
	}
}

void SString::reverse()
{
	wchar_t* tmp = new wchar_t[size];

	for (int i = 0; i < length; i += 1)
	{
		tmp[i] = string[length-1-i];
	}

	tmp[length] = '\0';
	SAFE_DELETE_ARRAY(string)
	string = tmp;
}

void SString::toUpperCase()
{
	for (int i = 0; i < length; i += 1)
	{
		if (string[i] >= 'a' && string[i] <= 'z')
		{
			string[i] = 'A'+(string[i]-'a');
		}
	}
}

void SString::toLowerCase()
{
	for (int i = 0; i < length; i += 1)
	{
		if (string[i] >= 'A' && string[i] <= 'Z')
		{
			string[i] = 'a'+(string[i]-'A');
		}
	}
}

void SString::printOn(ostream &output)
{
	for (int i = 0; i < length; i += 1)
	{
		char charOutput = string[i];
		output << charOutput;
	}
}

const char* SString::getCharString()
{
	SAFE_DELETE_ARRAY(buffer)
	
	if (isEmpty())
	{
		return (const char*)0;
	}

	buffer = new char[size];

	for (int i = 0; i < size; i += 1)
	{
		buffer[i] = string[i];
	}

	return buffer;
}

void SString::empty()
{
	SAFE_DELETE_ARRAY(string)

	length = 0;
	size = 0;
}

SString SString::subString(int start, int end)
{
	if (end == -1)
	{
		end = length;
	}

	if (end < 0 ||  end > length || 
		start < 0 || start > length || 
		start == end)
	{
		return "";
	}

	SString returnValue;

	for (int i = start; i < end; i += 1)
	{
		returnValue	+= string[i];
	}
	
	return returnValue;
}

void SString::insertAt(int index, SString aString)
{
	SString leftSide  = subString(0,index);
	SString rightSide = subString(index);
	
	operator =(leftSide + aString + rightSide);
}

void SString::remove(SString aString, int beginAt)
{
	bool found;

	for (int i = beginAt; i < length; i += 1)
	{
		found = true;

		for (int j = i,l = 0; j < i + aString.length; j += 1, l += 1)
		{
			if (j >= length)
			{
				found = false;
				break;
			}
			
			if (operator[](j) != aString[l])
			{
				found = false;
				break;
			}
		}

		if (found)
		{
			for (int k = i; k < i + aString.length; k += 1)
			{
				deleteAt(k);
				k -= 1;
				i -= 1;
			}
		}
	}
	
/*	int index;

	while ( (index = search(aString,beginAt) ) != -1)
	{
		for (int i = beginAt; i < beginAt+aString.length; i += 1)
		{
			deleteAt(i);
			i -= 1;
		}
	}*/
}

bool SString::isEmpty()
{
	return (length == 0);
}

void SString::deleteAt(int index)
{
	if (index < 0 || index >= length)
	{
		return;
	}

	SString leftSide = subString(0,index);
	SString rightSide = subString(index+1);

	operator =(leftSide+rightSide);
}

int SString::search(SString aString, int beginAt)
{
	bool found;

	for (int i = beginAt; i < length; i += 1)
	{
		found = true;

		for (int j = i,l = 0; j < i + aString.length; j += 1, l += 1)
		{
			if (j >= length)
			{
				found = false;
				break;
			}
			
			if (operator[](j) != aString[l])
			{
				found = false;
				break;
			}
		}
	
		if (found)
		{
			return i;
		}
	}

	return -1;
}

int SString::reverseSearch(SString aString, int beginAt)
{
	int index;
	
	index = getReverse().search(aString.getReverse(),beginAt);

	if (index != -1)
	{
		return length-aString.length-index;
	}
	else
	{
		return -1;
	}
}

SString SString::getReverse()
{
	SString returnValue;

	for (int i = length-1; i >= 0; i -= 1)
	{
		returnValue += string[i];
	}

	return returnValue;
}

///////////////////////////////////////////////////////////////////////
// Operatoren
///////////////////////////////////////////////////////////////////////

SString& SString::operator =(SString aString)
{
	SAFE_DELETE_ARRAY(string)

	length = aString.length;
	size = aString.size;
	string = new wchar_t[size];

	for (int i = 0; i < size; i += 1)
	{
		string[i] = aString.string[i];
	}

	return (*this);
}

SString SString::operator +(SString aString)
{
	SString returnValue = (*this);

	returnValue += aString;

	return returnValue;
}

void SString::operator +=(SString aString)
{
	int oldLength = length;

	length += aString.length;

	if (size != 0)
	{
		size += aString.size - 1;
	}
	else
	{
		size = aString.size;
	}
	
	
	wchar_t* tempBuffer = new wchar_t[size];

	for (int i = 0; i < oldLength; i += 1)
	{
		tempBuffer[i] = string[i];
	}
	
	for (int j = 0; j < aString.length; j += 1)
	{
		tempBuffer[j + oldLength] = aString.string[j];
	}

	tempBuffer[size-1] = '\0';

	SAFE_DELETE_ARRAY(string)

	string = tempBuffer;
}

bool SString::operator == (SString aString)
{
	if (length == aString.length)
	{
		for (int i = 0; i < length; i += 1)
		{
			if (string[i] != aString.string[i])
			{
				return false;
			}
		}

		return true;
	}
	else
	{
		return false;
	}
}

bool SString::operator != (SString aString)
{
	return (!operator == (aString));
}

bool SString::operator < (SString aString)
{
	return (wcscmp(string,aString.string) < 0);
}

bool SString::operator <= (SString aString)
{
	return ((*this) == aString || (*this) < aString);
}

bool SString::operator > (SString aString)
{
	return (wcscmp(string,aString.string) > 0);
}

bool SString::operator >= (SString aString)
{
	return ((*this) == aString || (*this) > aString);
}

wchar_t& SString::operator [](int index)
{
	if (index >= 0 && index < length)
	{
		return string[index];
	}
	else
	{
		return NULL_CHAR;
	}
}

///////////////////////////////////////////////////////////////////////////
// "Friends"
///////////////////////////////////////////////////////////////////////////

ostream& operator << (ostream& io,SString aString)
{
	aString.printOn(io);
	return io;
}

istream& operator >> (istream& io, SString& aString)
{
	char thisChar;
	aString.empty();
	
	for (;;)
	{
		io.get(thisChar);

		if (thisChar != '\n')
		{
			aString += thisChar;
		}
		else
		{
			break;
		}

	}
	return io;
}

